@@ -1,5 +1,5 @@
 #![feature(test)]
-#![cfg(all(any(target_arch = "x86", target_arch = "x86_64"), target_feature = "avx2"))]
+#![cfg(feature = "simd_avx2")]
 
 extern crate test;
 use test::{Bencher, black_box};
@@ -11,7 +11,8 @@ struct A([i16; L]);
 
 #[bench]
 fn bench_opt_prefix_scan(b: &mut Bencher) {
-    unsafe {
+    #[target_feature(enable = "avx2")]
+    unsafe fn inner(b: &mut Bencher) {
         let vec = A([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, 12, 13, 14, 11]);
         let vec = simd_load(vec.0.as_ptr() as *const Simd);
 
@@ -20,11 +21,13 @@ fn bench_opt_prefix_scan(b: &mut Bencher) {
             simd_prefix_scan_i16(black_box(vec), consts)
         });
     }
+    unsafe { inner(b); }
 }
 
 #[bench]
 fn bench_naive_prefix_scan(b: &mut Bencher) {
-    unsafe {
+    #[target_feature(enable = "avx2")]
+    unsafe fn inner(b: &mut Bencher) {
         let vec = A([0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, 12, 13, 14, 11]);
         let vec = simd_load(vec.0.as_ptr() as *const Simd);
 
@@ -33,4 +36,5 @@ fn bench_naive_prefix_scan(b: &mut Bencher) {
             simd_naive_prefix_scan_i16(black_box(vec), consts)
         });
     }
+    unsafe { inner(b); }
 }
