@@ -1,8 +1,3 @@
-#![cfg(any(
-        all(any(target_arch = "x86", target_arch = "x86_64"), target_feature = "avx2"),
-        all(target_arch = "wasm32", target_feature = "simd128")
-))]
-
 use bio::alignment::pairwise::*;
 use bio::alignment::{Alignment, AlignmentOperation};
 use bio::scores::blosum62;
