@@ -1,10 +1,6 @@
 #![feature(bench_black_box)]
-#![cfg(any(
-        all(any(target_arch = "x86", target_arch = "x86_64"), target_feature = "avx2"),
-        all(target_arch = "wasm32", target_feature = "simd128")
-))]
 
-#[cfg(not(target_arch = "wasm32"))]
+#[cfg(not(feature = "simd_wasm"))]
 use parasailors::{Matrix, *};
 
 use block_aligner::scan_block::*;
@@ -64,7 +60,7 @@ fn get_data(file_names: &[&str]) -> Vec<(Vec<u8>, Vec<u8>)> {
     res
 }
 
-#[cfg(not(target_arch = "wasm32"))]
+#[cfg(not(feature = "simd_wasm"))]
 fn bench_parasailors_aa_core(idx: usize, _trace: bool, _min_size: usize, _max_size: usize) -> (i32, Duration) {
     let file_data = get_data(&FILE_NAMES[idx]);
     let matrix = Matrix::new(MatrixType::Blosum62);
@@ -149,7 +145,7 @@ fn main() {
     let uc30_95_time = d.as_secs_f64();
     println!("ours (trace), uc30 0.95, 32-256, {}", uc30_95_time);
 
-    #[cfg(not(target_arch = "wasm32"))]
+    #[cfg(not(feature = "simd_wasm"))]
     {
         let d = time(bench_parasailors_aa_core, 0, false, 0, 0);
         let uc30_time = d.as_secs_f64();
