@@ -14,33 +14,43 @@ pub const ZERO: i16 = 1 << 14;
 pub const MIN: i16 = 0;
 
 // Non-temporal store to avoid cluttering cache with traces
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn store_trace(ptr: *mut TraceType, trace: TraceType) { _mm_stream_si32(ptr, trace); }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_adds_i16(a: Simd, b: Simd) -> Simd { _mm256_adds_epi16(a, b) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_subs_i16(a: Simd, b: Simd) -> Simd { _mm256_subs_epi16(a, b) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_max_i16(a: Simd, b: Simd) -> Simd { _mm256_max_epi16(a, b) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_cmpeq_i16(a: Simd, b: Simd) -> Simd { _mm256_cmpeq_epi16(a, b) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_cmpgt_i16(a: Simd, b: Simd) -> Simd { _mm256_cmpgt_epi16(a, b) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_blend_i8(a: Simd, b: Simd, mask: Simd) -> Simd { _mm256_blendv_epi8(a, b, mask) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_load(ptr: *const Simd) -> Simd { _mm256_load_si256(ptr) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_store(ptr: *mut Simd, a: Simd) { _mm256_store_si256(ptr, a) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_set1_i16(v: i16) -> Simd { _mm256_set1_epi16(v) }
 
@@ -74,6 +84,7 @@ macro_rules! simd_insert_i16 {
     };
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_movemask_i8(a: Simd) -> u32 { _mm256_movemask_epi8(a) as u32 }
 
@@ -115,6 +126,7 @@ macro_rules! simd_sr_i16 {
     };
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 unsafe fn simd_sl_i128(a: Simd, b: Simd) -> Simd {
     _mm256_permute2x128_si256(a, b, 0x03)
@@ -133,12 +145,14 @@ macro_rules! simd_sllz_i16 {
     };
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_broadcasthi_i16(v: Simd) -> Simd {
     let v = _mm256_shufflehi_epi16(v, 0b11111111);
     _mm256_permute4x64_epi64(v, 0b11111111)
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_slow_extract_i16(v: Simd, i: usize) -> i16 {
     debug_assert!(i < L);
@@ -151,6 +165,7 @@ pub unsafe fn simd_slow_extract_i16(v: Simd, i: usize) -> i16 {
     *a.0.as_ptr().add(i)
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_hmax_i16(v: Simd) -> i16 {
     let mut v2 = _mm256_max_epi16(v, _mm256_srli_si256(v, 2));
@@ -210,12 +225,14 @@ macro_rules! simd_prefix_hmax_i16 {
     };
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn simd_hargmax_i16(v: Simd, max: i16) -> usize {
     let v2 = _mm256_cmpeq_epi16(v, _mm256_set1_epi16(max));
     (simd_movemask_i8(v2).trailing_zeros() as usize) / 2
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 #[allow(non_snake_case)]
 #[allow(dead_code)]
@@ -232,6 +249,7 @@ pub unsafe fn simd_naive_prefix_scan_i16(R_max: Simd, (gap_cost, _gap_cost123456
     curr
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn get_gap_extend_all(gap: i16) -> Simd {
     _mm256_set_epi16(
@@ -244,6 +262,7 @@ pub unsafe fn get_gap_extend_all(gap: i16) -> Simd {
 
 pub type PrefixScanConsts = (Simd, Simd);
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn get_prefix_scan_consts(gap: i16) -> PrefixScanConsts {
     let gap_cost = _mm256_set1_epi16(gap);
@@ -256,6 +275,7 @@ pub unsafe fn get_prefix_scan_consts(gap: i16) -> PrefixScanConsts {
     (gap_cost, gap_cost12345678)
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 #[allow(non_snake_case)]
 pub unsafe fn simd_prefix_scan_i16(R_max: Simd, (gap_cost, gap_cost12345678): PrefixScanConsts) -> Simd {
@@ -282,6 +302,7 @@ pub unsafe fn simd_prefix_scan_i16(R_max: Simd, (gap_cost, gap_cost12345678): Pr
     _mm256_max_epi16(shift4, correct1)
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_lookup2_i16(lut1: HalfSimd, lut2: HalfSimd, v: HalfSimd) -> Simd {
     let a = _mm_shuffle_epi8(lut1, v);
@@ -291,11 +312,13 @@ pub unsafe fn halfsimd_lookup2_i16(lut1: HalfSimd, lut2: HalfSimd, v: HalfSimd)
     _mm256_cvtepi8_epi16(c)
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_lookup1_i16(lut: HalfSimd, v: HalfSimd) -> Simd {
     _mm256_cvtepi8_epi16(_mm_shuffle_epi8(lut, v))
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_lookup_bytes_i16(match_scores: HalfSimd, mismatch_scores: HalfSimd, a: HalfSimd, b: HalfSimd) -> Simd {
     let mask = _mm_cmpeq_epi8(a, b);
@@ -303,21 +326,27 @@ pub unsafe fn halfsimd_lookup_bytes_i16(match_scores: HalfSimd, mismatch_scores:
     _mm256_cvtepi8_epi16(c)
 }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_load(ptr: *const HalfSimd) -> HalfSimd { _mm_load_si128(ptr) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_loadu(ptr: *const HalfSimd) -> HalfSimd { _mm_loadu_si128(ptr) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_store(ptr: *mut HalfSimd, a: HalfSimd) { _mm_store_si128(ptr, a) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_sub_i8(a: HalfSimd, b: HalfSimd) -> HalfSimd { _mm_sub_epi8(a, b) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_set1_i8(v: i8) -> HalfSimd { _mm_set1_epi8(v) }
 
+#[target_feature(enable = "avx2")]
 #[inline]
 pub unsafe fn halfsimd_get_idx(i: usize) -> usize { i }
 
@@ -336,6 +365,7 @@ macro_rules! halfsimd_sr_i8 {
     };
 }
 
+#[target_feature(enable = "avx2")]
 #[allow(dead_code)]
 pub unsafe fn simd_dbg_i16(v: Simd) {
     #[repr(align(32))]
@@ -350,6 +380,7 @@ pub unsafe fn simd_dbg_i16(v: Simd) {
     println!();
 }
 
+#[target_feature(enable = "avx2")]
 #[allow(dead_code)]
 pub unsafe fn halfsimd_dbg_i8(v: HalfSimd) {
     #[repr(align(16))]
@@ -364,6 +395,7 @@ pub unsafe fn halfsimd_dbg_i8(v: HalfSimd) {
     println!();
 }
 
+#[target_feature(enable = "avx2")]
 #[allow(dead_code)]
 pub unsafe fn simd_assert_vec_eq(a: Simd, b: [i16; L]) {
     #[repr(align(32))]
@@ -374,6 +406,7 @@ pub unsafe fn simd_assert_vec_eq(a: Simd, b: [i16; L]) {
     assert_eq!(arr.0, b);
 }
 
+#[target_feature(enable = "avx2")]
 #[allow(dead_code)]
 pub unsafe fn halfsimd_assert_vec_eq(a: HalfSimd, b: [i8; L]) {
     #[repr(align(32))]
@@ -390,7 +423,8 @@ mod tests {
 
     #[test]
     fn test_prefix_scan() {
-        unsafe {
+        #[target_feature(enable = "avx2")]
+        unsafe fn inner() {
             #[repr(align(32))]
             struct A([i16; L]);
 
@@ -404,5 +438,6 @@ mod tests {
             let res = simd_prefix_scan_i16(simd_load(vec.0.as_ptr() as *const Simd), consts);
             simd_assert_vec_eq(res, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 15, 14, 13, 14, 13]);
         }
+        unsafe { inner(); }
     }
 }
